@@ -1,7 +1,15 @@
+"""
+TODO: 
+    - Rethink to/from_JSON_like
+    - Ensure we can reproduce the same RVE with random_seed specification
+"""
+
 import copy
 
 import numpy as np
 
+from damask_parse.utils import perpendicular_vectors
+
 
 from damask_parse.utils import get_coordinate_grid
 
@@ -165,6 +173,7 @@ class Particle:
 
     @classmethod
     def from_JSON_like(cls, dct, distribution=None):
+        dct = copy.deepcopy(dct)
         return cls(**dct, distribution=distribution)
 
     def _validate(self):
@@ -267,10 +276,20 @@ class ParticleDistribution:
             minor_axis_ratios = [1, 1]  # sphere
         if margins is None:
             margins = [0, 0, 0]
-        if major_axis_dir is None:
-            major_axis_dir = [1, 0, 0]
-        if major_plane_normal_dir is None:
-            major_plane_normal_dir = [0, 0, 1]
+
+        if major_axis_dir is None and major_plane_normal_dir is not None:
+            # find perpendicular major_axis_dir
+            major_plane_normal_dir = np.array(major_plane_normal_dir, dtype=float)
+            major_axis_dir = perpendicular_vectors(major_plane_normal_dir)
+        elif major_axis_dir is not None and major_plane_normal_dir is None:
+            # find perpendicular major_plane_normal_dir
+            major_axis_dir = np.array(major_axis_dir, dtype=float)
+            major_plane_normal_dir = perpendicular_vectors(major_axis_dir)
+        else:
+            if major_axis_dir is None:
+                major_axis_dir = [1, 0, 0]
+            if major_plane_normal_dir is None:
+                major_plane_normal_dir = [0, 0, 1]
 
         self.label = label
         self.number = number
@@ -308,6 +327,7 @@ class ParticleDistribution:
 
     @classmethod
     def from_JSON_like(cls, dct):
+        dct = copy.deepcopy(dct)
         particles = [Particle.from_JSON_like(i) for i in dct.pop('_particles', [])]
         obj = cls(_particles=particles, **dct)
         for particle in obj.particles:
@@ -431,7 +451,7 @@ class ParticleRVE:
         self.matrix_material_IDs = list(np.unique(self._material))
         self.particle_material_IDs = {None: []}  # keyed by particle distribution label
 
-        self.random_seed = random_seed
+        self.random_seed = random_seed or np.random.randint(0, 100_000)
         self.rng = np.random.default_rng(random_seed)
 
         self._add_particles(particles)
@@ -452,6 +472,7 @@ class ParticleRVE:
 
     @classmethod
     def from_JSON_like(cls, dct):
+        dct = copy.deepcopy(dct)
         particles = [Particle.from_JSON_like(i) for i in dct['particles']]
         particle_distributions = [
             ParticleDistribution.from_JSON_like(i)
@@ -679,7 +700,6 @@ class ParticleRVE:
             overlap_fraction = np.sum(overlap_bool) / np.sum(single_particle_RVE_voxels)
             count = 0
             while np.any(overlap_bool):
-                print(f'count: {count}')
                 if count > max_iter:
                     print(f'Cannot find suitable position for particle in {max_iter} '
                           f'iterations. Skipping.')
