@@ -966,13 +966,13 @@ def validate_orientations(orientations):
             # "normalise" to scalar-vector convention:
             quaternions = np.roll(quaternions, 1, axis=1)
             quats_comp_order = 'scalar-vector'
-        
+
     if quats_comp_order not in ALLOWED_QUAT_ORDER:
         msg = (f'Quaternion component order key `quat_component_ordering` must be '
-                f'specified as either "scalar-vector" or "vector-scalar". Actual '
-                f'value was: "{quats_comp_order}".')
+               f'specified as either "scalar-vector" or "vector-scalar". Actual '
+               f'value was: "{quats_comp_order}".')
         raise ValueError(msg)
-        
+
     # To ensure maximum precision of quaternions, cast to longdouble (although note that
     # precision is system-dependent):
     quaternions = quaternions.astype(np.longdouble)
@@ -1427,7 +1427,7 @@ def get_volume_element_materials(volume_element, homog_schemes=None, phases=None
     mat_const_idx = get_material_constituent_idx(const_mat_idx)
 
     all_quats = volume_element['orientations']['quaternions']
-    
+
     quat_comp_order = volume_element['orientations'].get('quat_component_ordering')
     if quat_comp_order != 'scalar-vector':
         msg = (f'Quaternion component ordering (`quat_component_ordering`) should be '
@@ -1555,6 +1555,7 @@ def prepare_material_yaml_data(mat_dat):
 
     return mat_dat_fmt
 
+
 def get_coordinate_grid(size, grid_size):
     """Get the coordinates of the element centres of a uniform grid."""
 
@@ -1570,3 +1571,62 @@ def get_coordinate_grid(size, grid_size):
     coords += element_size / 2
 
     return coords, element_size
+
+
+def perpendicular_vectors(vec, axis=-1):
+    """
+    Get 3-vectors perpendicular to a given set of 3-vectors.
+
+    Parameters
+    ----------
+    vec : ndarray
+        Array of vectors.
+    axis : int, optional
+        The axis along which the 3-vectors are defined.
+
+    Returns
+    -------
+    perp_vec : ndarray
+        Array of the same shape as input array `vec`, where each output vector
+        is perpendicular to its corresponding input vector.
+
+    """
+
+    if vec.shape[axis] != 3:
+        raise ValueError('Size of dimension `axis` ({}) along `vec` must be 3,'
+                         ' but is {}.'.format(axis, vec.shape[axis]))
+
+    # Reshape:
+    vec = np.swapaxes(vec, -1, axis)
+
+    # Return array will have the same shape as input
+    perp_vec = np.ones_like(vec) * np.nan
+
+    # Find where first component magnitudes are larger than last:
+    a_gt_c = np.abs(vec[..., 0]) > np.abs(vec[..., 2])
+    a_notgt_c = np.logical_not(a_gt_c)
+
+    # Make bool index arrays
+    a_gt_c_0 = np.zeros_like(perp_vec, dtype=bool)
+    a_gt_c_0[..., 0] = a_gt_c
+    a_gt_c_1 = np.roll(a_gt_c_0, shift=1, axis=-1)
+    a_gt_c_2 = np.roll(a_gt_c_1, shift=1, axis=-1)
+
+    a_notgt_c_0 = np.zeros_like(perp_vec, dtype=bool)
+    a_notgt_c_0[..., 0] = a_notgt_c
+    a_notgt_c_1 = np.roll(a_notgt_c_0, shift=1, axis=-1)
+    a_notgt_c_2 = np.roll(a_notgt_c_1, shift=1, axis=-1)
+
+    # Set each component of the output vectors:
+    perp_vec[a_gt_c_0] = vec[a_gt_c][..., 2]
+    perp_vec[a_gt_c_1] = 0
+    perp_vec[a_gt_c_2] = -vec[a_gt_c][..., 0]
+
+    perp_vec[a_notgt_c_0] = 0
+    perp_vec[a_notgt_c_1] = vec[a_notgt_c][..., 2]
+    perp_vec[a_notgt_c_2] = -vec[a_notgt_c][..., 1]
+
+    # Reshape to original shape:
+    perp_vec = np.swapaxes(perp_vec, -1, axis)
+
+    return perp_vec
